package org.apache.commons.lang3.tuple;

package .tmp;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;
import org.apache.commons.lang3.builder.CompareToBuilder;
import org.apache.commons.lang3.tuple.ImmutablePair;
import org.apache.commons.lang3.tuple.Pair;

public abstract class Pair<L, R> implements Map.Entry<L, R>, Comparable<Pair<L, R>>, Serializable {
  private static final long serialVersionUID = 4954918890077093841L;
  
  public static final Pair<?, ?>[] EMPTY_ARRAY = (Pair<?, ?>[])new PairAdapter[0];
  
  public static <L, R> Pair<L, R>[] emptyArray() {
    return (Pair<L, R>[])EMPTY_ARRAY;
  }
  
  public static <L, R> Pair<L, R> of(L left, R right) {
    return ImmutablePair.of(left, right);
  }
  
  public static <L, R> Pair<L, R> of(Map.Entry<L, R> pair) {
    return ImmutablePair.of(pair);
  }
  
  public int compareTo(Pair<L, R> other) {
    return (new CompareToBuilder()).append(getLeft(), other.getLeft())
      .append(getRight(), other.getRight()).toComparison();
  }
  
  public boolean equals(Object obj) {
    if (obj == this)
      return true; 
    if (obj instanceof Map.Entry) {
      Map.Entry<?, ?> other = (Map.Entry<?, ?>)obj;
      return (Objects.equals(getKey(), other.getKey()) && 
        Objects.equals(getValue(), other.getValue()));
    } 
    return false;
  }
  
  public final L getKey() {
    return getLeft();
  }
  
  public R getValue() {
    return getRight();
  }
  
  public int hashCode() {
    return Objects.hashCode(getKey()) ^ Objects.hashCode(getValue());
  }
  
  public String toString() {
    return "(" + getLeft() + ',' + getRight() + ')';
  }
  
  public String toString(String format) {
    return String.format(format, new Object[] { getLeft(), getRight() });
  }
  
  public abstract L getLeft();
  
  public abstract R getRight();
}
